package cz.cvut.fel.pr1.airports;

import de.micromata.opengis.kml.v_2_2_0.Document;
import de.micromata.opengis.kml.v_2_2_0.Kml;

import java.io.File;
import java.io.FileNotFoundException;

/**
 * Simple generator of KML files using {@link Airports} class.
 * @author Ondrej Hrstka
 */
public class KmlGenerator {

    private final Airports airports;

    public KmlGenerator(Airports airports) {
        this.airports = airports;
    }

    /**
     * Generates output into 'airports.kml' file.
     */
    public void generateKml() {
        Kml kml = new Kml();
        final Document doc = kml.createAndSetDocument();

        generatePlacemarks(doc);
        generateLines(doc);

        try {
            kml.marshal(new File("airports.kml"));
        } catch (FileNotFoundException e) {
            System.err.println("Error while writing out kml:");
            e.printStackTrace();
            System.exit(1);
        }
    }

    private void generatePlacemarks(Document doc) {
        for (int i = 0; i < Identificators.ALL.length; i++) {
            final String id = Identificators.ALL[i];

            final double lat = airports.latitudeOfAirport(id);
            final double lon = airports.longitudeOfAirport(id);
            final String name = airports.nameOfAirport(id);

            doc.createAndAddPlacemark().
                    withName(id + " - " + name).
                    createAndSetPoint().
                    addToCoordinates(lon, lat);
        }
    }

    private void generateLines(Document doc) {
        final String pragueId = Identificators.ALL[0]; // Prague is first
        final double pragueLat = airports.latitudeOfAirport(pragueId);
        final double pragueLon = airports.longitudeOfAirport(pragueId);


        for (int i = 1; i < Identificators.ALL.length; i++) {
            final String id = Identificators.ALL[i];

            final double lat = airports.latitudeOfAirport(id);
            final double lon = airports.longitudeOfAirport(id);

            final double distance = airports.distanceBetweenAirports(pragueId, id, false);

            doc.createAndAddPlacemark().
                    withName(String.format("%f km", distance)).
                    withDescription(String.format("%f km", distance)).
                    createAndSetLineString().
                    withTessellate(true).
                    addToCoordinates(pragueLon, pragueLat).
                    addToCoordinates(lon, lat);
        }
    }

}
